@@ -63,7 +63,7 @@ class socket_base_t : public own_t,
 
   public:
     //  Returns false if object is not a socket.
-    bool check_tag ();
+    bool check_tag () const;
 
     //  Returns whether the socket is thread-safe.
     bool is_thread_safe () const;
@@ -73,7 +73,7 @@ class socket_base_t : public own_t,
     create (int type_, zmq::ctx_t *parent_, uint32_t tid_, int sid_);
 
     //  Returns the mailbox associated with this socket.
-    i_mailbox *get_mailbox ();
+    i_mailbox *get_mailbox () const;
 
     //  Interrupt blocking call if the socket is stuck in one.
     //  This function can be called from a different thread!
@@ -82,9 +82,9 @@ class socket_base_t : public own_t,
     //  Interface for communication with the API layer.
     int setsockopt (int option_, const void *optval_, size_t optvallen_);
     int getsockopt (int option_, void *optval_, size_t *optvallen_);
-    int bind (const char *addr_);
-    int connect (const char *addr_);
-    int term_endpoint (const char *addr_);
+    int bind (const char *endpoint_uri_);
+    int connect (const char *endpoint_uri_);
+    int term_endpoint (const char *endpoint_uri_);
     int send (zmq::msg_t *msg_, int flags_);
     int recv (zmq::msg_t *msg_, int flags_);
     void add_signaler (signaler_t *s_);
@@ -120,25 +120,29 @@ class socket_base_t : public own_t,
 
     int monitor (const char *endpoint_, int events_);
 
-    void event_connected (const std::string &addr_, zmq::fd_t fd_);
-    void event_connect_delayed (const std::string &addr_, int err_);
-    void event_connect_retried (const std::string &addr_, int interval_);
-    void event_listening (const std::string &addr_, zmq::fd_t fd_);
-    void event_bind_failed (const std::string &addr_, int err_);
-    void event_accepted (const std::string &addr_, zmq::fd_t fd_);
-    void event_accept_failed (const std::string &addr_, int err_);
-    void event_closed (const std::string &addr_, zmq::fd_t fd_);
-    void event_close_failed (const std::string &addr_, int err_);
-    void event_disconnected (const std::string &addr_, zmq::fd_t fd_);
-    void event_handshake_failed_no_detail (const std::string &addr_, int err_);
-    void event_handshake_failed_protocol (const std::string &addr_, int err_);
-    void event_handshake_failed_auth (const std::string &addr_, int err_);
-    void event_handshake_succeeded (const std::string &addr_, int err_);
+    void event_connected (const std::string &endpoint_uri_, zmq::fd_t fd_);
+    void event_connect_delayed (const std::string &endpoint_uri_, int err_);
+    void event_connect_retried (const std::string &endpoint_uri_,
+                                int interval_);
+    void event_listening (const std::string &endpoint_uri_, zmq::fd_t fd_);
+    void event_bind_failed (const std::string &endpoint_uri_, int err_);
+    void event_accepted (const std::string &endpoint_uri_, zmq::fd_t fd_);
+    void event_accept_failed (const std::string &endpoint_uri_, int err_);
+    void event_closed (const std::string &endpoint_uri_, zmq::fd_t fd_);
+    void event_close_failed (const std::string &endpoint_uri_, int err_);
+    void event_disconnected (const std::string &endpoint_uri_, zmq::fd_t fd_);
+    void event_handshake_failed_no_detail (const std::string &endpoint_uri_,
+                                           int err_);
+    void event_handshake_failed_protocol (const std::string &endpoint_uri_,
+                                          int err_);
+    void event_handshake_failed_auth (const std::string &endpoint_uri_,
+                                      int err_);
+    void event_handshake_succeeded (const std::string &endpoint_uri_, int err_);
 
     //  Query the state of a specific peer. The default implementation
     //  always returns an ENOTSUP error.
-    virtual int get_peer_state (const void *identity_,
-                                size_t identity_size_) const;
+    virtual int get_peer_state (const void *routing_id_,
+                                size_t routing_id_size_) const;
 
   protected:
     socket_base_t (zmq::ctx_t *parent_,
@@ -150,7 +154,8 @@ class socket_base_t : public own_t,
     //  Concrete algorithms for the x- methods are to be defined by
     //  individual socket types.
     virtual void xattach_pipe (zmq::pipe_t *pipe_,
-                               bool subscribe_to_all_ = false) = 0;
+                               bool subscribe_to_all_ = false,
+                               bool locally_initiated_ = false) = 0;
 
     //  The default implementation assumes there are no specific socket
     //  options for the particular socket type. If not so, override this
@@ -166,11 +171,6 @@ class socket_base_t : public own_t,
     virtual bool xhas_in ();
     virtual int xrecv (zmq::msg_t *msg_);
 
-    //  Returns the credential for the peer from which we have received
-    //  the last message. If no message has been received yet,
-    //  the function returns empty credential.
-    virtual const blob_t &get_credential () const;
-
     //  i_pipe_events will be forwarded to these functions.
     virtual void xread_activated (pipe_t *pipe_);
     virtual void xwrite_activated (pipe_t *pipe_);
@@ -184,21 +184,21 @@ class socket_base_t : public own_t,
     //  Delay actual destruction of the socket.
     void process_destroy ();
 
-    // Next assigned name on a zmq_connect() call used by ROUTER and STREAM socket types
-    std::string connect_routing_id;
-
   private:
     // test if event should be sent and then dispatch it
-    void event (const std::string &addr_, intptr_t fd_, int type_);
+    void event (const std::string &endpoint_uri_, intptr_t value_, int type_);
 
     // Socket event data dispatch
-    void monitor_event (int event_, intptr_t value_, const std::string &addr_);
+    void monitor_event (int event_,
+                        intptr_t value_,
+                        const std::string &endpoint_uri_) const;
 
     // Monitor socket cleanup
     void stop_monitor (bool send_monitor_stopped_event_ = true);
 
     //  Creates new endpoint ID and adds the endpoint to the map.
-    void add_endpoint (const char *addr_, own_t *endpoint_, pipe_t *pipe_);
+    void
+    add_endpoint (const char *endpoint_uri_, own_t *endpoint_, pipe_t *pipe_);
 
     //  Map of open endpoints.
     typedef std::pair<own_t *, pipe_t *> endpoint_pipe_t;
@@ -206,7 +206,17 @@ class socket_base_t : public own_t,
     endpoints_t _endpoints;
 
     //  Map of open inproc endpoints.
-    typedef std::multimap<std::string, pipe_t *> inprocs_t;
+    class inprocs_t
+    {
+      public:
+        void emplace (const char *endpoint_uri_, pipe_t *pipe_);
+        int erase_pipes (const std::string &endpoint_uri_str_);
+        void erase_pipe (pipe_t *pipe_);
+
+      private:
+        typedef std::multimap<std::string, pipe_t *> map_t;
+        map_t _inprocs;
+    };
     inprocs_t _inprocs;
 
     //  To be called after processing commands or invoking any command
@@ -229,15 +239,17 @@ class socket_base_t : public own_t,
     bool _destroyed;
 
     //  Parse URI string.
-    int
-    parse_uri (const char *uri_, std::string &protocol_, std::string &address_);
+    static int
+    parse_uri (const char *uri_, std::string &protocol_, std::string &path_);
 
     //  Check whether transport protocol, as specified in connect or
     //  bind, is available and compatible with the socket type.
-    int check_protocol (const std::string &protocol_);
+    int check_protocol (const std::string &protocol_) const;
 
     //  Register the pipe with this socket.
-    void attach_pipe (zmq::pipe_t *pipe_, bool subscribe_to_all_ = false);
+    void attach_pipe (zmq::pipe_t *pipe_,
+                      bool subscribe_to_all_ = false,
+                      bool locally_initiated_ = false);
 
     //  Processes commands sent to this socket (if any). If timeout is -1,
     //  returns only after at least one command was processed.
@@ -253,6 +265,9 @@ class socket_base_t : public own_t,
 
     void update_pipe_options (int option_);
 
+    std::string resolve_tcp_addr (std::string endpoint_uri_,
+                                  const char *tcp_address_);
+
     //  Socket's mailbox object.
     i_mailbox *_mailbox;
 
@@ -300,6 +315,53 @@ class socket_base_t : public own_t,
     socket_base_t (const socket_base_t &);
     const socket_base_t &operator= (const socket_base_t &);
 };
+
+class routing_socket_base_t : public socket_base_t
+{
+  protected:
+    routing_socket_base_t (class ctx_t *parent_, uint32_t tid_, int sid_);
+    ~routing_socket_base_t ();
+
+    // methods from socket_base_t
+    virtual int
+    xsetsockopt (int option_, const void *optval_, size_t optvallen_);
+    virtual void xwrite_activated (pipe_t *pipe_);
+
+    // own methods
+    std::string extract_connect_routing_id ();
+    bool connect_routing_id_is_set () const;
+
+    struct out_pipe_t
+    {
+        pipe_t *pipe;
+        bool active;
+    };
+
+    void add_out_pipe (blob_t routing_id_, pipe_t *pipe_);
+    bool has_out_pipe (const blob_t &routing_id_) const;
+    out_pipe_t *lookup_out_pipe (const blob_t &routing_id_);
+    const out_pipe_t *lookup_out_pipe (const blob_t &routing_id_) const;
+    void erase_out_pipe (pipe_t *pipe_);
+    out_pipe_t try_erase_out_pipe (const blob_t &routing_id_);
+    template <typename Func> bool any_of_out_pipes (Func func_)
+    {
+        bool res = false;
+        for (out_pipes_t::iterator it = _out_pipes.begin ();
+             it != _out_pipes.end () && !res; ++it) {
+            res |= func_ (*it->second.pipe);
+        }
+
+        return res;
+    }
+
+  private:
+    //  Outbound pipes indexed by the peer IDs.
+    typedef std::map<blob_t, out_pipe_t> out_pipes_t;
+    out_pipes_t _out_pipes;
+
+    // Next assigned name on a zmq_connect() call used by ROUTER and STREAM socket types
+    std::string _connect_routing_id;
+};
 }
 
 #endif
